@@ -3,7 +3,7 @@
 /* 
  * Library for Posix async read operations with hints.
  * Author: Don Capps
- * Company: Hewlett Packard
+ * Company: Iozone
  * Date: 4/24/1998
  *
  * Two models are supported.  First model is a replacement for read() where the async
@@ -96,13 +96,15 @@
 
 #include <sys/types.h>
 #include <aio.h>
-#if defined(solaris) || defined(linux) || defined(SCO_Unixware_gcc)
+#if defined(solaris) || defined(linux) || defined(SCO_Unixware_gcc) || defined(__NetBSD__)
 #else
 #include <sys/timers.h>
 #endif
 #include <sys/errno.h>
 #include <unistd.h>
+#ifndef bsd4_4
 #include <malloc.h>
+#endif
 #ifdef VXFS
 #include <sys/fs/vx_ioctl.h>
 #endif
@@ -117,7 +119,7 @@
 #include <stdlib.h>
 #endif
 
-#if ((defined(solaris) && defined(__LP64__)) || defined(__s390x__))
+#if (defined(solaris) && defined(__LP64__)) || defined(__s390x__) || defined(FreeBSD) || defined(__NetBSD__)
 /* If we are building for 64-bit Solaris, all functions that return pointers
  * must be declared before they are used; otherwise the compiler will assume
  * that they return ints and the top 32 bits of the pointer will be lost,
@@ -148,7 +150,7 @@ extern int one;
  * cache, pointed to by async_init(gc) will be of
  * this structure type.
  */
-char version[] = "Libasync Version $Revision: 3.19 $";
+char version[] = "Libasync Version $Revision: 3.25 $";
 struct cache_ent {
 	struct aiocb myaiocb;			/* For use in small file mode */
 #ifdef _LARGEFILE64_SOURCE 
@@ -193,6 +195,7 @@ struct cache_ent *alloc_cache();
 struct cache_ent *incache();
 void async_init();
 void end_async();
+int async_suspend();
 int async_read();
 void takeoff_cache();
 void del_cache();
@@ -261,6 +264,33 @@ struct cache *gc;
 	free((void *)gc);
 }
 
+/***********************************************/
+/* Wait for a request to finish                */
+/***********************************************/
+int
+async_suspend(struct cache_ent *ce)
+{
+#ifdef _LARGEFILE64_SOURCE 
+#ifdef __LP64__
+	const struct aiocb * const cblist[1] = {&ce->myaiocb};
+#else
+	const struct aiocb64 * const cblist[1] = {&ce->myaiocb64};
+#endif
+#else
+	const struct aiocb * const cblist[1] = {&ce->myaiocb};
+#endif
+
+#ifdef _LARGEFILE64_SOURCE 
+#ifdef __LP64__
+	return aio_suspend(cblist, 1, NULL);
+#else
+	return aio_suspend64(cblist, 1, NULL);
+#endif
+#else
+	return aio_suspend(cblist, 1, NULL);
+#endif
+}
+
 /*************************************************************************
  * This routine is a generic async reader assist funtion. It takes
  * the same calling parameters as read() but also extends the
@@ -320,15 +350,21 @@ long long depth;
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
 		while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
+		{
+			async_suspend(ce);
+		}
 #else
 		while((ret=aio_error64(&ce->myaiocb64))== EINPROGRESS)
+		{
+			async_suspend(ce);
+		}
 #endif
 #else
 		while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
-#endif
 		{
-			;
+			async_suspend(ce);
 		}
+#endif
 		if(ret)
 		{
 			printf("aio_error 1: ret %d %d\n",ret,errno);
@@ -467,15 +503,21 @@ out:
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
 		while((ret=aio_error(&first_ce->myaiocb))== EINPROGRESS)
+		{
+			async_suspend(first_ce);
+		}
 #else
 		while((ret=aio_error64(&first_ce->myaiocb64))== EINPROGRESS)
+		{
+			async_suspend(first_ce);
+		}
 #endif
 #else
 		while((ret=aio_error(&first_ce->myaiocb))== EINPROGRESS)
-#endif
 		{
-			;
+			async_suspend(first_ce);
 		}
+#endif
 		if(ret)
 			printf("aio_error 2: ret %d %d\n",ret,errno);
 #ifdef _LARGEFILE64_SOURCE 
@@ -825,15 +867,21 @@ long long depth;
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
 		while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
+		{
+			async_suspend(ce);
+		}
 #else
 		while((ret=aio_error64(&ce->myaiocb64))== EINPROGRESS)
+		{
+			async_suspend(ce);
+		}
 #endif
 #else
 		while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
-#endif
 		{
-			;
+			async_suspend(ce);
 		}
+#endif
 		if(ret)
 			printf("aio_error 3: ret %d %d\n",ret,errno);
 #ifdef _LARGEFILE64_SOURCE 
@@ -1003,15 +1051,21 @@ out:
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
 		while((ret=aio_error(&first_ce->myaiocb))== EINPROGRESS)
+		{
+			async_suspend(first_ce);
+		}
 #else
 		while((ret=aio_error64(&first_ce->myaiocb64))== EINPROGRESS)
+		{
+			async_suspend(first_ce);
+		}
 #endif
 #else
 		while((ret=aio_error(&first_ce->myaiocb))== EINPROGRESS)
-#endif
 		{
-			;
+			async_suspend(first_ce);
 		}
+#endif
 		if(ret)
 			printf("aio_error 4: ret %d %d\n",ret,errno);
 #ifdef _LARGEFILE64_SOURCE 
@@ -1386,15 +1440,21 @@ struct cache *gc;
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
 	while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
+	{
+		async_suspend(ce);
+	}
 #else
 	while((ret=aio_error64(&ce->myaiocb64))== EINPROGRESS)
+	{
+		async_suspend(ce);
+	}
 #endif
 #else
 	while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
-#endif
 	{
-		;
+		async_suspend(ce);
 	}
+#endif
 	if(ret)
 	{
 		printf("aio_error 5: ret %d %d\n",ret,errno);
